// file      : libbrep/build.cxx -*- C++ -*-
// copyright : Copyright (c) 2014-2017 Code Synthesis Ltd
// license   : MIT; see accompanying LICENSE file

#include <libbrep/build.hxx>

namespace brep
{
  // build_state
  //
  string
  to_string (build_state s)
  {
    switch (s)
    {
    case build_state::untested: return "untested";
    case build_state::testing:  return "testing";
    case build_state::tested:   return "tested";
    }

    return string (); // Should never reach.
  }

  build_state
  to_build_state (const string& s)
  {
         if (s == "untested") return build_state::untested;
    else if (s == "testing")  return build_state::testing;
    else if (s == "tested")   return build_state::tested;
    else throw invalid_argument ("invalid build state '" + s + "'");
  }

  // build
  //
  build::
  build (string pnm, version pvr,
         string cfg,
         string tnm, version tvr,
         string mnm, string msm)
      : id (package_id (move (pnm), pvr), move (cfg), tvr),
        package_name (id.package.name),
        package_version (move (pvr)),
        configuration (id.configuration),
        toolchain_name (move (tnm)),
        toolchain_version (move (tvr)),
        state (build_state::testing),
        timestamp (timestamp_type::clock::now ()),
        forced (false),
        machine (move (mnm)),
        machine_summary (move (msm))
  {
  }
}
